/*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.jnosql.bdbje;

import java.io.File;
import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import jakarta.nosql.Value;
import jakarta.nosql.keyvalue.BucketManager;
import jakarta.nosql.keyvalue.KeyValueEntity;

import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.DatabaseEntry;
import com.sleepycat.je.Environment;
import com.sleepycat.je.EnvironmentConfig;
import com.sleepycat.je.OperationStatus;

import dk.vajhoej.jnosql.Deserialize;
import dk.vajhoej.jnosql.Serialize;

class BDBJEBucketManager implements BucketManager {
	private Environment env;
	private Database db;
	private boolean prefix;
	private String bucket;
	private Serialize to;
	private Deserialize from;
	private DatabaseEntry keyWrap(Object key) {
		String keystr;
		if(key instanceof byte[]) {
			keystr = new String((byte[])key);
		} else if(key instanceof char[]) {
			keystr = new String((char[])key);
		} else {
			keystr = key.toString();
		}
		if(prefix) {
			keystr = bucket + ":" + keystr;
		}
		return new DatabaseEntry(keystr.getBytes());
	}
	public BDBJEBucketManager(String dbdir, String dbnam, boolean prefix, String bucket, Serialize to, Deserialize from) {
        EnvironmentConfig envcfg = new EnvironmentConfig();
        envcfg.setAllowCreateVoid(true);
        env = new Environment(new File(dbdir), envcfg);
        DatabaseConfig dbcfg = new DatabaseConfig();
        dbcfg.setAllowCreate(true);
        db = env.openDatabase(null, dbnam, dbcfg);
        this.prefix = prefix;
        this.bucket = bucket;
		this.to = to;
		this.from = from;
	}
	@Override
	public <K, V> void put(K key, V val) {
		try {
			db.put(null,  keyWrap(key), new DatabaseEntry(to.convert(val)));
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	@Override
	public void put(KeyValueEntity kve) {
		put(kve.getKey(), kve.getValue());
	}
	@Override
	public void put(Iterable<KeyValueEntity> kves) {
		for(KeyValueEntity kve : kves) {
			put(kve);
		}
	}
	@Override
	public void put(KeyValueEntity kve, Duration exp) {
		throw new UnsupportedOperationException("BDB-JE does not support expiration");
	}
	@Override
	public void put(Iterable<KeyValueEntity> kves, Duration exp) {
		throw new UnsupportedOperationException("BDB-JE does not support expiration");
	}
	@Override
	public <K> Optional<Value> get(K key) {
        DatabaseEntry dbe = new DatabaseEntry();
        if(db.get(null, keyWrap(key), dbe, null) == OperationStatus.SUCCESS) {
        	try {
				return Optional.of(Value.of(from.convert(dbe.getData())));
			} catch (ClassNotFoundException e) {
				return Optional.empty();
			} catch (IOException e) {
				return Optional.empty();
			}
        } else {
        	return Optional.empty();
        }
	}
	@Override
	public <K> Iterable<Value> get(Iterable<K> keys) {
		List<Value> res = new ArrayList<Value>();
		for(K key : keys) {
			Optional<Value> temp = get(key);
			if(temp.isPresent()) {
				res.add(temp.get());
			}
		}
		return res;
	}
	@Override
	public <K> void delete(K key) {
		db.delete(null, keyWrap(key));
	}
	@Override
	public <K> void delete(Iterable<K> keys) {
		for(K key : keys) {
			delete(key);
		}
	}
	@Override
	public void close() {
        db.close();
        env.close();
	}
}
